#---
# Excerpted from "Programming Phoenix 1.4",
# published by The Pragmatic Bookshelf.
# Copyrights apply to this code. It may not be used to create training material,
# courses, books, articles, and the like. Contact us if you are in doubt.
# We make no guarantees that this code is fit for any purpose.
# Visit http://www.pragmaticprogrammer.com/titles/phoenix14 for more book information.
#---
defmodule InfoSysTest do
  use ExUnit.Case
  alias InfoSys.Result

  defmodule TestBackend do
    def name(), do: "Wolfram"
    
    def compute("result", _opts) do
      [%Result{backend: __MODULE__, text: "result"}]
    end
    def compute("none", _opts) do
      []
    end
    def compute("timeout", _opts) do
      Process.sleep(:infinity)
    end
    def compute("boom", _opts) do
      raise "boom!"
    end
  end

  @tag :capture_log
  test "compute/2 discards backend errors" do
    assert InfoSys.compute("boom", backends: [TestBackend]) == []
  end

  test "compute/2 with timeout returns no results" do 
    results = InfoSys.compute("timeout", backends: [TestBackend], timeout: 10)
    assert results == []
  end

  test "compute/2 with backend results" do 
    assert [%Result{backend: TestBackend, text: "result"}] =
           InfoSys.compute("result", backends: [TestBackend])
  end

  test "compute/2 with no backend results" do 
    assert [] = InfoSys.compute("none", backends: [TestBackend])
  end
end
